package net.wanji.datacenter.service.impl;

import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.extern.slf4j.Slf4j;
import net.wanji.common.utils.tool.JacksonUtils;
import net.wanji.datacenter.cache.CrossInfoCache;
import net.wanji.datacenter.common.PhaseEmptyEnum;
import net.wanji.datacenter.mapper.CrossStrategyResultMapper;
import net.wanji.datacenter.pojo.dto.ImbalanceAlgorithmDTO;
import net.wanji.datacenter.pojo.dto.PhaseEmptyDataDTO;
import net.wanji.datacenter.pojo.entity.CrossStrategyResultEntity;
import net.wanji.datacenter.service.CrossStrategyOptService;
import org.apache.dubbo.common.utils.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2024/11/23 16:14
 */
@Service
@Slf4j
public class CrossStrategyOptServiceImpl implements CrossStrategyOptService {

    @Resource
    private CrossStrategyResultMapper crossStrategyResultMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void savePhaseEmpty(String message) throws Exception {
        try {
            ObjectMapper mapper = JacksonUtils.getInstance();
            PhaseEmptyDataDTO phaseEmptyDataDTO = mapper.readValue(message, PhaseEmptyDataDTO.class);
            if (Objects.nonNull(phaseEmptyDataDTO)) {
                PhaseEmptyDataDTO.Idx idx = phaseEmptyDataDTO.getIdx();
                String crossId = idx.getCtrId();
                Long curStamp = idx.getCurStamp();
                Integer countDown = idx.getCountDown();
                Integer rtnType = idx.getRtnType();
                // rtnType大于0时，未执行
                Long signalMachineStamp = idx.getSignalMachineStamp();
                Long createStamp = idx.getCreateStamp();
                Date date = new Date(curStamp);
                CrossStrategyResultEntity entity = new CrossStrategyResultEntity();
                entity.setCrossId(crossId);
                entity.setSignalMachineStamp(String.valueOf(signalMachineStamp));
                // 绿灯空放
                entity.setCurrentAlgo(1);
                entity.setRequestTime(date);
                entity.setIssueTime(date);
                entity.setInsertTime(new Date(createStamp));
                entity.setCountDown(countDown);
                entity.setExtendTime(countDown);
                entity.setEventId(idx.getEventId());
                if (rtnType < 0) {
                    if (Objects.nonNull(idx.getResponseCode()) && !Objects.equals(-1, idx.getResponseCode())) {
                        entity.setResponseCode(idx.getResponseCode());
                    } else {
                        entity.setResponseCode(200);
                    }
                    entity.setResponseContent(idx.getResponseContent());
                } else {
                    if (Objects.nonNull(idx.getResponseCode()) && !Objects.equals(-1, idx.getResponseCode())) {
                        entity.setResponseCode(idx.getResponseCode());
                    } else {
                        entity.setResponseCode(500);
                    }
                    String desc = PhaseEmptyEnum.getDesc(rtnType);
                    entity.setResponseContent(desc);
                }
                entity.setRtnType(rtnType);
                String name = CrossInfoCache.getName(crossId);
                entity.setCrossName(name);
                entity.setEmptyDir(PhaseEmptyEnum.getDir(rtnType));
                entity.setEmptyTurn(PhaseEmptyEnum.getTurn(rtnType));
                if (Objects.nonNull(date)) {
                    String parse = DateUtil.format(date, "yyyyMMdd");
                    entity.setDt(Integer.valueOf(parse));
                } else {
                    Date current = new Date();
                    String parse = DateUtil.format(current, "yyyyMMdd");
                    entity.setDt(Integer.valueOf(parse));
                }
                crossStrategyResultMapper.insert(entity);
            }
        } catch (Exception e) {
            log.error("相位空放优化结果入库异常：", e);
            throw new Exception(e);
        }

    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void save(ImbalanceAlgorithmDTO dto) {
        try {
            if (Objects.equals(-1, dto.getControl_method())) {
                String crossId = dto.getCrossId();
                SimpleDateFormat format = new SimpleDateFormat("yyyyMMdd");
                Date issueTime = dto.getIssueTime();
                Date date = new Date();
                String currentDate = format.format(date);
                LambdaQueryWrapper<CrossStrategyResultEntity> query = new LambdaQueryWrapper<CrossStrategyResultEntity>();
                query.eq(CrossStrategyResultEntity::getCrossId, crossId);
                query.eq(CrossStrategyResultEntity::getDt, currentDate);
                query.orderByDesc(CrossStrategyResultEntity::getIssueTime);
                query.last("limit 1");
                List<CrossStrategyResultEntity> results = crossStrategyResultMapper.selectList(query);
                if (!CollectionUtils.isEmpty(results) && results.size() == 1) {
                    CrossStrategyResultEntity crossStrategyResultEntity = results.get(0);
                    long offset = date.getTime() - issueTime.getTime();
                    int duration = (int) (offset / 1000);
                    log.warn("当前失衡路口：{}，当前路口发生事件：{}， 计算时间差：{}", crossId, issueTime, duration);
                    crossStrategyResultEntity.setDuration(duration + crossStrategyResultEntity.getDuration());
                    crossStrategyResultMapper.updateById(crossStrategyResultEntity);
                }
                return;
            }
            ObjectMapper mapper = JacksonUtils.getInstance();
            CrossStrategyResultEntity crossStrategyResultEntity = new CrossStrategyResultEntity();
            crossStrategyResultEntity.setCrossId(dto.getPlatformId());
            crossStrategyResultEntity.setCrossName(dto.getCrossName());
            crossStrategyResultEntity.setCurrentAlgo(dto.getCurrentAlgo());
            crossStrategyResultEntity.setRequestTime(dto.getRequestTime());
            Date issueTime = dto.getIssueTime();
            crossStrategyResultEntity.setIssueTime(dto.getIssueTime());
            crossStrategyResultEntity.setResponseCode(dto.getResponseCode());
            crossStrategyResultEntity.setTimingPlan(mapper.writeValueAsString(dto.getTimingPlan()));
            crossStrategyResultEntity.setInsertTime(dto.getInsertTime());
            crossStrategyResultEntity.setRtnType(dto.getRtnType());
            crossStrategyResultEntity.setExtendTime(dto.getExtendTime());
            crossStrategyResultEntity.setBlockRegion(dto.getBlockRegion() + 1);
            crossStrategyResultEntity.setBlockType(dto.getBlockType());
            crossStrategyResultEntity.setCountDown(dto.getCountDown());
            crossStrategyResultEntity.setVideoStamp(dto.getVideoStamp());
            crossStrategyResultEntity.setSignalMachineStamp(dto.getSignalMachineStamp());
            crossStrategyResultEntity.setControlDir(dto.getControlDir());
            crossStrategyResultEntity.setData(mapper.writeValueAsString(dto.getData()));
            crossStrategyResultEntity.setResponseContent(dto.getResponseContent());
            crossStrategyResultEntity.setDuration(dto.getControl_duration());
            crossStrategyResultEntity.setEventId(dto.getEventId());
            if (Objects.nonNull(issueTime)) {
                String parse = DateUtil.format(issueTime, "yyyyMMdd");
                crossStrategyResultEntity.setDt(Integer.valueOf(parse));
            } else {
                Date date = new Date();
                String parse = DateUtil.format(date, "yyyyMMdd");
                crossStrategyResultEntity.setDt(Integer.valueOf(parse));
            }
            crossStrategyResultMapper.insert(crossStrategyResultEntity);
        } catch (Exception e) {
            log.error("失衡优化策略数据处理失败：", e);
            throw new RuntimeException(e);
        }
    }
}
